function [A,F]=assem(assKey, parm, U, Ug, VERT, SIMP);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% FEMBIF = < Finite Element Methods for Bifurcation >
%% Copyright (C) 1997  Michael Holst
%%
%% This program is free software; you can redistribute it and/or modify it
%% under the terms of the GNU General Public License as published by the Free
%% Software Foundation; either version 2 of the License, or (at your option)
%% any later version.
%%
%% This program is distributed in the hope that it will be useful, but WITHOUT
%% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
%% FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
%% more details.
%%
%% You should have received a copy of the GNU General Public License along
%% with this program; if not, write to the Free Software Foundation, Inc.,
%% 675 Mass Ave, Cambridge, MA 02139, USA.
%%
%% Typing "help 0license" should list the GNU GPL to the screen.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Prototype:
%
%    [A,F]=assem(assKey, parm, U, Ug, VERT, SIMP);
%
% Purpose:
%
%    Finite element matrix assembly.
%
% Input:
%
%    (N  = number of vertices in mesh)
%    (L  = number of simplices in mesh)
%
%    assKey = assembly path
%    parm   = pde parameters
%    U      = zeros(N,1); current finite element solution
%    Ug     = zeros(N,1); essential/dirichlet boundary function
%    VERT   = zeros(N,6); vertices, (x,y,z)-coordinates of all vertices
%    SIMP   = zeros(L,9); simplices, vertices numbers given counter-clockwise
%
% Output:
%
%    assKey==0 ==>    A  = zeros(N,N) = linear stiffness matrix
%                     F  = zeros(N,1) = linear load vector
%
%    assKey==1 ==>    A  = zeros(N,N) = nonlinear tangent stiffness matrix
%                     F  = zeros(N,1) = linear load vector
%
%    assKey==2 ==>    A  = zeros(N,N) = nonlinear tangent stiffness matrix
%                     F  = zeros(N,1) = nonlinear residual vector
%
%    assKey==3 ==>    A  = zeros(N,N) = (not constructed)
%                     F  = zeros(N,1) = nonlinear residual vector
%
% Other Variables:
%
%    T  = number of vertices in a single element
%    Q  = number of quadrature points in the interior of the master element
%    QE = number of quadrature points along the master element edges
%
%    INTPT   = zeros(Q,2)  = master element volume integration points
%    W       = zeros(Q,1)  = master element volume integration weights
%    PHI     = zeros(T,Q)  = values of local basis funcs at integ pts
%    PHIX    = zeros(T,Q)  = values of x-derivs of local basis funcs integ pts
%    PHIY    = zeros(T,Q)  = values of y-derivs of local basis funcs integ pts
%
%    INTPTE0 = zeros(QE,2) = type (1,2) EDGE integration points
%    WE0     = zeros(QE,1) = type (1,2) EDGE integration weights
%    PHIE0   = zeros(T,QE) = type (1,2) basis function values at EDGE integ pts
%
%    INTPTE1 = zeros(QE,2) = type (2,0) EDGE integration points
%    WE1     = zeros(QE,1) = type (2,0) EDGE integration weights
%    PHIE1   = zeros(T,QE) = type (2,0) basis function values at EDGE integ pts
%
%    INTPTE2 = zeros(QE,2) = type (0,1) EDGE integration points
%    WE2     = zeros(QE,1) = type (0,1) EDGE integration weights
%    PHIE2   = zeros(T,QE) = type (0,1) basis function values at EDGE integ pts
%
% Author:   Michael Holst 050497
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% initialization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   %%% first recover various problem dimensions.
   [N,six]  = size(VERT);
   [L,nine] = size(SIMP);
   T=3;

   %%% get master element info.
   [INTPT,W,PHI,PHIX,PHIY, ...
    INTPTE0,WE0,PHIE0, ...
    INTPTE1,WE1,PHIE1, ...
    INTPTE2,WE2,PHIE2] = mastr(0);
   [Q,two]  = size(INTPT);
   [QE,two] = size(INTPTE0);

   %%% initialize global stiffness matrix and global load vector.
   A = spalloc(N,N,0);  
   F = zeros(N,1);

   %%% get coordinates of vertices of each element.
   Xi0(1:L,1) = VERT(SIMP(1:L,1),1);
   Xi1(1:L,1) = VERT(SIMP(1:L,2),1);
   Xi2(1:L,1) = VERT(SIMP(1:L,3),1);
   Yi0(1:L,1) = VERT(SIMP(1:L,1),2);
   Yi1(1:L,1) = VERT(SIMP(1:L,2),2);
   Yi2(1:L,1) = VERT(SIMP(1:L,3),2);

   %%% build triangle affine transformations (from master el to arbitrary).
   f00(1:L,1) = Xi1(1:L,1) - Xi0(1:L,1);
   f01(1:L,1) = Xi2(1:L,1) - Xi0(1:L,1);
   f10(1:L,1) = Yi1(1:L,1) - Yi0(1:L,1);
   f11(1:L,1) = Yi2(1:L,1) - Yi0(1:L,1);
   b0(1:L,1)  = Xi0(1:L,1);
   b1(1:L,1)  = Yi0(1:L,1);

   %%% build the triangle jacobians.
   D (1:L,1) = abs( f00 .* f11 - f01 .* f10 );
   Di(1:L,1) = 1.0 ./ D(1:L,1);

   %%% build the edge jacobians.
   DE = zeros(L,3);
   DE(1:L,1) = (1/sqrt(2)) ...
             * sqrt( (Xi2(1:L,1) - Xi1(1:L,1)) .* (Xi2(1:L,1) - Xi1(1:L,1)) ...
             + (Yi2(1:L,1) - Yi1(1:L,1)) .* (Yi2(1:L,1) - Yi1(1:L,1)) );
   DE(1:L,2) = sqrt( (Xi0(1:L,1) - Xi2(1:L,1)) .* (Xi0(1:L,1) - Xi2(1:L,1)) ...
             + (Yi0(1:L,1) - Yi2(1:L,1)) .* (Yi0(1:L,1) - Yi2(1:L,1)) );
   DE(1:L,3) = sqrt( (Xi0(1:L,1) - Xi1(1:L,1)) .* (Xi0(1:L,1) - Xi1(1:L,1)) ...
             + (Yi0(1:L,1) - Yi1(1:L,1)) .* (Yi0(1:L,1) - Yi1(1:L,1)) );

   %%% build inverse triangle transformations (from arbitrary el to master).
   g00(1:L,1) =  Di(1:L,1) .* f11(1:L,1);
   g01(1:L,1) = -Di(1:L,1) .* f01(1:L,1);
   g10(1:L,1) = -Di(1:L,1) .* f10(1:L,1);
   g11(1:L,1) =  Di(1:L,1) .* f00(1:L,1);

   %%% edge types
   etype = facetype(SIMP(1:L,5));

   %%% at each r loop iter, compute el stiffness matrix and el load vec entries
   for r=1:T

      %%% initialize element load entry fr
      fr(1:L,1) = zeros(L,1);

      %%% at each s loop iter, compute el stiffness matrix entries
      for s=1:T

         %%% initialize element matrix entry ars 
         ars(1:L,1) = zeros(L,1);

         %%% go thru volume quad pts m=1:Q for contrib to element matrix/vector
         for m=1:Q

            %%% mask with weight and indicating which edge are neumann edge
            WM = zeros(L,1);
            WM(1:L,1) = W(m) * D(1:L,1);

            %%% get quad pts by mapping master Gauss pts to our element
            xm(1:L,1)=f00(1:L,1)*INTPT(m,1)+f01(1:L,1)*INTPT(m,2)+b0(1:L,1);
            ym(1:L,1)=f10(1:L,1)*INTPT(m,1)+f11(1:L,1)*INTPT(m,2)+b1(1:L,1);

            %%% evaluate function at the current quadrature point m
            for q=1:T
               p(q) = PHI(q,m);
               px(1:L,q) = g00(1:L,1) * PHIX(q,m) + g10(1:L,1) * PHIY(q,m);
               py(1:L,q) = g01(1:L,1) * PHIX(q,m) + g11(1:L,1) * PHIY(q,m);
            end

            %%% current solution at quadrature points in all simplices
            UU(1:L,1) = p(1) * U( SIMP(1:L,1), 1 ) ...
                      + p(2) * U( SIMP(1:L,2), 1 ) ...
                      + p(3) * U( SIMP(1:L,3), 1 );
            UUX(1:L,1) = px(1:L,1) .* U( SIMP(1:L,1), 1 ) ...
                       + px(1:L,2) .* U( SIMP(1:L,2), 1 ) ...
                       + px(1:L,3) .* U( SIMP(1:L,3), 1 );
            UUY(1:L,1) = py(1:L,1) .* U( SIMP(1:L,1), 1 ) ...
                       + py(1:L,2) .* U( SIMP(1:L,2), 1 ) ...
                       + py(1:L,3) .* U( SIMP(1:L,3), 1 );

            %%% essential/dirichlet function at quad points in all simplices
            UUg(1:L,1) = p(1) * Ug( SIMP(1:L,1), 1 ) ...
                       + p(2) * Ug( SIMP(1:L,2), 1 ) ...
                       + p(3) * Ug( SIMP(1:L,3), 1 );
            UUgX(1:L,1) = px(1:L,1) .* Ug( SIMP(1:L,1), 1 ) ...
                        + px(1:L,2) .* Ug( SIMP(1:L,2), 1 ) ...
                        + px(1:L,3) .* Ug( SIMP(1:L,3), 1 );
            UUgY(1:L,1) = py(1:L,1) .* Ug( SIMP(1:L,1), 1 ) ...
                        + py(1:L,2) .* Ug( SIMP(1:L,2), 1 ) ...
                        + py(1:L,3) .* Ug( SIMP(1:L,3), 1 );

            %%% do the load vector while we are here...
            if (s==1)

               %%% evaluate nonlinear function at quadrature points
               evalKey = 0;
               thetam = fu_v(evalKey,parm,L, xm,ym, ...
                  (UU+UUg), (UUX+UUgX), (UUY+UUgY), ...
                  p(r), px(1:L,r),py(1:L,r));

               %%% do quad using weights W
               fr(1:L,1) = fr(1:L,1) + WM(1:L,1) .* thetam(1:L,1);
            end;

            %%% evaluate bilinear form at quadrature points
            evalKey = 0;
            thetam = dfu_wv(evalKey,parm,L, xm,ym, ...
                            (UU+UUg), (UUX+UUgX), (UUY+UUgY), ...
                            p(s),px(1:L,s),py(1:L,s), ...
                            p(r),px(1:L,r),py(1:L,r));

            %%% do quad using weights W
            ars(1:L,1) = ars(1:L,1) + WM(1:L,1) .* thetam(1:L,1);
         end %%% the "m=1:Q" loop

         %%% go thru edge quad pts m=1:QE for contrib to element matrix/vector
         for m=1:QE
            
            %%% handle contributions to each of the possible edges
            for edge=1:T

               %%% use the appropriate quadrature info for particular edge
               if (edge==1)
                  INTE = INTPTE0;
                  PHIE = PHIE0;
                  WE   = WE0;
               elseif (edge==2)
                  INTE = INTPTE1;
                  PHIE = PHIE1;
                  WE   = WE1;
               elseif (edge==3)
                  INTE = INTPTE2;
                  PHIE = PHIE2;
                  WE   = WE2;
               end

               %%% mask with weight and indicating which edge are neumann edge
               WEM = zeros(L,1);
               WEM(1:L,1) = WE(m) * DE(1:L,edge) ...
                          .* (etype(1:L,edge)==2*ones(L,1));

               %%% Get quad pts by mapping master Gauss pts to our element
               xm(1:L,1)=f00(1:L,1)*INTE(m,1)+f01(1:L,1)*INTE(m,2)+b0(1:L,1);
               ym(1:L,1)=f10(1:L,1)*INTE(m,1)+f11(1:L,1)*INTE(m,2)+b1(1:L,1);

               %%% evaluate function at the current quadrature point m
               for q=1:T
                  p(q) = PHIE(q,m);
                  px(1:L,q) = zeros(L,1);
                  py(1:L,q) = zeros(L,1);
               end

               %%% current solution at quadrature points in all simplices
               UU(1:L,1)  = p(1) * U( SIMP(1:L,1), 1 ) ...
                          + p(2) * U( SIMP(1:L,2), 1 ) ...
                          + p(3) * U( SIMP(1:L,3), 1 );
               UUX(1:L,1) = zeros(L,1);
               UUY(1:L,1) = zeros(L,1);

               %%% essential/dirichlet function at quad points in all simplices
               UUg(1:L,1)  = p(1) * Ug( SIMP(1:L,1), 1 ) ...
                           + p(2) * Ug( SIMP(1:L,2), 1 ) ...
                           + p(3) * Ug( SIMP(1:L,3), 1 );
               UUgX(1:L,1) = zeros(L,1);
               UUgY(1:L,1) = zeros(L,1);

               %%% neumann/edge term (do the load vector while here...)
               if (s==1)        
                   evalKey = 1;
                   thetam = fu_v(evalKey,parm,L, xm,ym, ...
                      (UU+UUg), (UUX+UUgX), (UUY+UUgY), ...
                      p(r), px(1:L,r),py(1:L,r));
                   fr(1:L,1) = fr(1:L,1) + WEM(1:L,1) .* thetam(1:L,1);
               end

               %%% robin/volume term (evaluate bilinear form at quad point)
               evalKey = 1;
               thetam = dfu_wv(evalKey,parm,L, xm,ym, ...
                            (UU+UUg), (UUX+UUgX), (UUY+UUgY), ...
                            p(s),px(1:L,s),py(1:L,s), ...
                            p(r),px(1:L,r),py(1:L,r));
               ars(1:L,1) = ars(1:L,1) + WEM(1:L,1) .* thetam(1:L,1);
            end %%% the "edge=1:T" loop
         end %%% the "m=1:QE" loop

         %%% Add contribution to global stiffness matrix.
         i(1:L,1)  = SIMP(1:L,r);
         j(1:L,1)  = SIMP(1:L,s);
         ik(1:L,1) = VERT( SIMP(1:L,r), 4 );
         jk(1:L,1) = VERT( SIMP(1:L,s), 4 );
         for l=1:L
            if ( (ik(l) ~= 1) & (jk(l) ~= 1) )
               A(i(l),j(l)) = A(i(l),j(l)) + ars(l);
            else
               if (ik(l) == 1)
                  A(i(l),i(l)) = 1.0;
               end
               if (jk(l) == 1)
                  A(j(l),j(l)) = 1.0;
               end
            end
         end
      end %%% the "s=1:T" loop

      %%% Add contribution to global load vector
      i(1:L,1)  = SIMP(1:L,r);
      ik(1:L,1) = VERT( SIMP(1:L,r), 4 );
      for l=1:L
         if (ik(l) ~= 1)
            F(i(l)) = F(i(l)) - fr(l);
         else
            if (assKey <= 1)
               F(i(l)) = 0.;
            else
               F(i(l)) = 0.;
            end
         end
      end
   end %%% the "r=1:T" loop

