function [VERT_F,SIMP_F] = refin(VERT,SIMP);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% FEMBIF = < Finite Element Methods for Bifurcation >
%% Copyright (C) 1997  Michael Holst
%%
%% This program is free software; you can redistribute it and/or modify it
%% under the terms of the GNU General Public License as published by the Free
%% Software Foundation; either version 2 of the License, or (at your option)
%% any later version.
%%
%% This program is distributed in the hope that it will be useful, but WITHOUT
%% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
%% FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
%% more details.
%%
%% You should have received a copy of the GNU General Public License along
%% with this program; if not, write to the Free Software Foundation, Inc.,
%% 675 Mass Ave, Cambridge, MA 02139, USA.
%%
%% Typing "help 0license" should list the GNU GPL to the screen.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Prototype:
%
%    [VERT_F,SIMP_F] = refin(VERT,SIMP);
%
% Purpose:
%
%    Uniformly refine the finite element mesh and produce a linear 
%    prolongation operator which relates functions on the two meshes.
%
% Input:
%
%    (N = number of vertices in (coarse) mesh)
%    (L = number of simplices in (coarse) mesh)
%
%    VERT = size(N,5) ==> vertex coordinates
%
%              VERT(:,1) == x-coordinates of all vertices
%              VERT(:,2) == y-coordinates of all vertices
%              VERT(:,3) == z-coordinates of all vertices (3d case & manifold)
%              VERT(:,4) == integer information bitmap
%              VERT(:,5) == pointer to first simplex
%              VERT(:,6) == pointer to first edge
%
%    SIMP = size(L,9)  = simplices; vertex nums given counter-clockwise
%
%              SIMP(:,1) == 1st-vertex of all simplices
%              SIMP(:,2) == 2nd-vertex of all simplices
%              SIMP(:,3) == 3rd-vertex of all simplices
%              SIMP(:,4) == 4th-vertex of all simplices (3d case)
%              SIMP(:,5) == integer information bitmap
%              SIMP(:,6) == next simplex in ring of simplices about vertex 1
%              SIMP(:,7) == next simplex in ring of simplices about vertex 2
%              SIMP(:,8) == next simplex in ring of simplices about vertex 3
%              SIMP(:,9) == next simplex in ring of simplices about vertex 4
%
% Output:
%
%    VERT_F = list of vertices in the refined mesh
%    SIMP_F = conformally refined set of simplices
%
% Temporary edges:  used to avoid searching algorithms; the structure is:
%
%    K  = number of temporary edges in mesh
%    EDGE == size(K,6) ==> edge info, decoded as:
%
%              EDGE(:,1) == 1st vertex making up edge
%              EDGE(:,2) == 2nd vertex making up edge
%              EDGE(:,3) == integer information bitmap
%              EDGE(:,4) == midpoint vertex number
%              EDGE(:,5) == next edge number in ring of edges about vertex 1
%              EDGE(:,6) == next edge number in ring of edges about vertex 2
%
% Some useful mappings:
%
%    emap[4][4] ==> Convenient (invertible, pairs-to-singles) edge mapping:
%
%        Mapping              Edge (vtx pair)  Edge (number)
%        ------------------   --------------   -----------
%        i+j+(i/3)+(j/3)-1:   (0,1) or (1,0)   0 \           \
%                             (0,2) or (2,0)   1 |-- 2D Case |
%                             (1,2) or (2,1)   2 /           |--- 3D Case
%                             (0,3) or (3,0)   3             |
%                             (1,3) or (3,1)   4             |
%                             (2,3) or (3,2)   5             /
%
%    emapI[6][2] ==> Singles-to-pairs edge mapping; the inverse of "emap".
%
%    pmap[6][6] ==> The edge permutation map; this mapping allows
%                   us to map any of the three/six refinement
%                   scenarios onto the canonical one.
%                   Each row represents the appropriate edge
%                   number mappings for each of the three/six
%                   cases.
%
%    vmap[6][4] ==> The vertex permutation map; this mapping allows
%                   us to map any of the three/six refinement
%                   scenarios onto the canonical one.
%                   Each row represents the appropriate vertex
%                   number mappings for each of the three/six
%                   cases.
%
% Author:   Michael Holst 050497
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% doit
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% some useful mappings to have around
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   emap  = [ 0,  1,  2,  4;
             1,  0,  3,  5;
             2,  3,  0,  6;
             4,  5,  6,  0 ];

   emapI = [ 1,  2;
             1,  3;
             2,  3;
             1,  4;
             2,  4;
             3,  4 ];

   pmap = [ 1, 2, 3, 4, 5, 6;
            2, 3, 1, 6, 4, 5;
            3, 1, 2, 5, 6, 4;
            4, 1, 5, 2, 6, 3;
            5, 4, 1, 6, 3, 2;
            6, 2, 4, 3, 5, 1 ];

   vmap = [ 1, 2, 3, 4;
            3, 1, 2, 4;
            2, 3, 1, 4;
            1, 4, 2, 3;
            4, 2, 1, 3;
            3, 4, 1, 2 ];

%%% recover array dimensions

   [N,six]  = size(VERT);
   [L,nine] = size(SIMP);
   T=3;

%%% create the auxilary simplex ring structures

   VERT(:,5:6) = zeros(N,2);
   SIMP(:,6:9) = zeros(L,4);
   for l=1:L
      for i=1:T
         oldFirstS = VERT( SIMP(l,i) ,5);
         VERT( SIMP(l,i) ,5) = l;
         SIMP(l,i+5) = oldFirstS;
      end
   end

%%% create the auxilary (temporary) edge ring structures

   KK = 0;
   EDGE = [];

%%% the refinement loop

   NN = N;
   LL = L;
   whichQ = 2;
   numRef = -1;
   while (numRef ~= 0)
      %%% reset refinement count
      numRef = 0;

      %%% set the current que
      if (whichQ == 2)
         whichQ = 1;
      else
         if (whichQ == 1)
            whichQ = 2;
         end
      end

      %%% loop over the elements
      for l=1:L
         %%% see if we need to refine this guy
         refq = getq(SIMP(l,5));
         if (refq(whichQ) ~= 0)

            %%% we have something to do
            numRef = numRef+1;

            %%% face/vertex info for the T vertices/faces making up this simplex
            vn = zeros(T);
            vs = zeros(T);
            ve = zeros(T);
            v = zeros(3,T);
            for i=1:T
               vn(i)  = SIMP(l,i);
               vs(i)  = VERT(vn(i),5);
               ve(i)  = VERT(vn(i),6);
               v(i,:) = VERT(vn(i),1:3);
            end
            etype = facetype(SIMP(l,5));

            %%% remove simplex from rings (prepare for reusing as first child)
            for i=1:T
               firstS = VERT( vn(i), 5 );
               if (l == firstS)
                  VERT( vn(i), 5) = SIMP(l,i+5);
               else
                  prevS = firstS;
                  ii = -1;
                  for j=1:T
                     if ( SIMP(prevS, j) == vn(i) ) ii = j; end
                  end
                  if (ii<0) fprintf('REFIN: Problem!\n'); end;
                  prevI = ii;
                  nextS = SIMP(prevS,prevI+5);
                  if (ii<0) fprintf('REFIN: Problem!\n'); end;
                  while ( nextS ~= 0 )
                     if ( nextS == l )
                        SIMP( prevS, prevI+5) = SIMP( l, i+5 );
                        nextS = 0;
                     else
                        prevS = nextS;
                        ii = -1;
                        for j=1:T
                           if ( SIMP(nextS, j) == vn(i) ) ii = j; end
                        end
                        prevI = ii;
                        nextS = SIMP( prevS, prevI+5 );
                        if (ii<0) fprintf('REFIN: Problem!\n'); end;
                     end
                  end
               end
            end
      
            %%% get existing vertices or make new ones as needed
            for i=1:T
               for j=i+1:T
                  edg = ve(i);
                  found = 0;
                  while ((found == 0) & (edg ~= 0))
                     edgDat = EDGE(edg,:);
                     if ( ((edgDat(1) == vn(i)) & (edgDat(2) == vn(j)) ) ...
                        | ((edgDat(1) == vn(j)) & (edgDat(2) == vn(i)) ) )
                        found = 1;
                     else
                        ii=-1;
                        if ( EDGE(edg,1) == vn(i) ) ii = 1; end;
                        if ( EDGE(edg,2) == vn(i) ) ii = 2; end;
                        if (ii<0) fprintf('REFIN: Problem!\n'); end;
                        edg = EDGE( edg, ii+4);
                     end
                  end
   
                  if (found == 0)
                     %%% determine type for new vertex
                     k=j+1;
                     if (k>T) k=1; end;
                     if (k==i) k=k+1; end;
                     vt_NEW = 0;
                     if ((etype(k)==1) | (etype(4)==1))
                        vt_NEW = 1;
                     else
                        if ((etype(k)>=2) | (etype(4)>=2))
                           vt_NEW = max(etype(k), etype(4));
                        end
                     end

                     %%% create the new vertex
                     NN=NN+1;
                     vn_NEW(i,j) = NN;
                     v_NEW = ( v(i,:) + v(j,:) ) / 2.0;
                     VERT = [ VERT ; v_NEW, vt_NEW, 0,0 ];

                     %%% create the marking edge
                     KK=KK+1;
                     firstEi = VERT( vn(i) ,6);
                     firstEj = VERT( vn(j) ,6);
                     VERT( vn(i) , 6 ) = KK;
                     VERT( vn(j) , 6 ) = KK;
                     EDGE = [ EDGE ; vn(i), vn(j), 0, vn_NEW(i,j), ...
                              firstEi, firstEj ];
                  else
                     vn_NEW(i,j) = edgDat(4);
                  end
               end
            end

            %%% new simplex facetype encoding
            s1typ = 1*0 + 16*etype(2) + 256*etype(3) + 4096*0;
            s2typ = 1*0 + 16*etype(3) + 256*etype(1) + 4096*0;
            s3typ = 1*0 + 16*etype(1) + 256*etype(2) + 4096*0;
            s4typ = 1*0 + 16*0        + 256*0        + 4096*0;

            %%% child simplex 1; create/add to rings (reuse parent structure)
            SIMP(l,:) = [ vn(1),  vn_NEW(1,2), vn_NEW(1,3), 0, ...
               s1typ, 0,0,0,0 ];
            for i=1:T
               oldFirstS = VERT( SIMP(l,i) ,5);
               VERT( SIMP(l,i) ,5) = l;
               SIMP(l,i+5) = oldFirstS;
            end

            %%% child simplex 2; create/add to rings (new simplex location)
            SIMP = [ SIMP ; vn(2),  vn_NEW(2,3), vn_NEW(1,2), 0, ...
               s2typ, 0,0,0,0 ];
            LL=LL+1;
            for i=1:T
               oldFirstS = VERT( SIMP(LL,i) ,5);
               VERT( SIMP(LL,i) ,5) = LL;
               SIMP(LL,i+5) = oldFirstS;
            end

            %%% child simplex 3; create/add to rings (new simplex location)
            SIMP = [ SIMP ; vn(3),  vn_NEW(1,3), vn_NEW(2,3), 0, ...
               s3typ, 0,0,0,0 ];
            LL=LL+1;
            for i=1:T
               oldFirstS = VERT( SIMP(LL,i) ,5);
               VERT( SIMP(LL,i) ,5) = LL;
               SIMP(LL,i+5) = oldFirstS;
            end

            %%% child simplex 4; create/add to rings (new simplex location)
            SIMP = [ SIMP ; vn_NEW(1,2), vn_NEW(2,3), vn_NEW(1,3), 0, ...
               s4typ, 0,0,0,0];
            LL=LL+1;
            for i=1:T
               oldFirstS = VERT( SIMP(LL,i) ,5);
               VERT( SIMP(LL,i) ,5) = LL;
               SIMP(LL,i+5) = oldFirstS;
            end
         end
      end
   end

%%% check new sizes

   [N,six]  = size(VERT);
   [L,nine] = size(SIMP);
   if (L ~= LL) fprintf('REFIN: Problem!\n'); end;
   if (N ~= NN) fprintf('REFIN: Problem!\n'); end;

%%% reinitialize the simplex and edge rings

   VERT(:,5:6) = zeros(N,2);
   SIMP(:,6:9) = zeros(L,4);
   EDGE = [];

%%% return the results

   VERT_F = VERT;
   SIMP_F = SIMP;

